import java.applet.Applet;
import java.io.*;
import java.util.*;
import javax.security.auth.x500.X500Principal;
import java.security.*;
import java.security.cert.*;
/*import org.bouncycastle.jce.provider.*;
import org.bouncycastle.jce.*;*/


/* Applet that evaluates the X509 certificates */
public class CertApplet extends Applet {


	private X509Certificate x509Cert=null;
	
	//For converting the boolean[] to human readable infos
	private String[] keyUsageString = new String[] {"digitalSignature","nonRepudiation",
					"keyEncipherment","dataEncipherment","keyAgreement","keyCertSign",
					"cRLSign","encipherOnly","decipherOnly" };
	
	//For converting the index number of a general name to human readable infos	
	private String[] generalNames = new String[] {"otherName","rfc822Name",
					"dNSName","x400Address","directoryName","ediPartyName",
					"uniformResourceIdentifier","iPAddress","registeredID" };
	
	private HashMap<String,String> extendedKeyUsage= new HashMap<String,String>();

	public CertApplet(){
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.1", "TLS web server authentication");
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.2", "TLS web client authentication");
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.3", "Code signing");
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.4", "E-mail protection");
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.5", "IP security end system");
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.6", "IP security tunnel termination");
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.7", "IP security user");
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.8", "Timestamping");
		extendedKeyUsage.put("1.3.6.1.5.5.7.3.9", "OCSPstamping");
		/*TODO: Add more entries*/
	}
	
	public void setCertificate(String pem){
		try{
			InputStream in=new ByteArrayInputStream(pem.getBytes());
			//CertificateFactory fact=CertificateFactory.getInstance("X.509","BC");
			CertificateFactory fact=CertificateFactory.getInstance("X.509");
			x509Cert=(X509Certificate)fact.generateCertificate(in);
		}catch(Exception ex){	//TODO: verbessern
			ex.printStackTrace();
			x509Cert=null;
		}
	}
	
	public String getIssuerDN(){
		try{
			if(x509Cert!=null){
				return x509Cert.getIssuerDN().toString();
			}else{
				return "no value found";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			ex.printStackTrace();
			return "no value found";
		}
	}
	
	public String getSubjectDN(){
		try{
			if(x509Cert!=null){
				return x509Cert.getSubjectDN().toString();
			}else{
				return "no value found";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			ex.printStackTrace();
			return "no value found";
		}
	}
	public String getNotBefore(){
		try{
			if(x509Cert!=null){
				return x509Cert.getNotBefore().toString();
			}else{
				return "no value found";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			ex.printStackTrace();
			return "no value found";
		}
	}
	public String getNotAfter(){
		try{
			if(x509Cert!=null){
				return x509Cert.getNotAfter().toString();
			}else{
				return "no value found";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			ex.printStackTrace();
			return "no value found";
		}
	}
	public boolean isValid(){
		try{
			if(x509Cert!=null){
				x509Cert.checkValidity();
				return true;
			}else{
				return false;
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			ex.printStackTrace();
			return false;
		}
	}
	public String getSerialNumber(){
		try{
			if(x509Cert!=null){
				return x509Cert.getSerialNumber().toString();
			}else{
				return "no value found";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			ex.printStackTrace();
			return "no value found";
		}
	}
	
	public String getVersion(){
		try{
			if(x509Cert!=null){
				return x509Cert.getVersion()+"";
			}else{
				return "no value found";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			ex.printStackTrace();
			return "no value found";
		}
	}
	
	
	public String getPublicKeyAlgorithm(){
		try{
			if(x509Cert!=null){
				return x509Cert.getPublicKey().getAlgorithm();
			}else{
				return "no value found";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			ex.printStackTrace();
			return "no value found";
		}
	}
	
	public String getBasicConstraint(){
		try{
			if(x509Cert!=null){
				if(x509Cert.getBasicConstraints()>0){
					return "true ("+x509Cert.getBasicConstraints()+")";
				}else{
					return "false";
				}
			}else{
				return "no value found";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			return "no value found";
		}
	}
	
	public String getKeyUsage(){
		try{
			if(x509Cert!=null){
				String allKeyUsage="";
				if(x509Cert.getKeyUsage().length>0){
					for(int i=0;i<x509Cert.getKeyUsage().length;i++){
						if(x509Cert.getKeyUsage()[i]){
								if(allKeyUsage!=""){allKeyUsage+=", ";}
							allKeyUsage+=keyUsageString[i];
						}
					}
				}
				return allKeyUsage;
			}else{
				return "-";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			return "-";
		}
	}
	
	public String getSubjectAlternativeNames(){
		try{
			if(x509Cert!=null){
					String allNames="";
					Collection<List<?>> names=x509Cert.getSubjectAlternativeNames();
					for(List<?> l:names){
						if(allNames!=""){allNames+=", ";}
						allNames+=generalNames[Integer.parseInt(l.get(0).toString())]+": "+l.get(1);
					}
					return allNames;
			}else{
				return "-";
			}
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			return "-";
		}
	}
	
	public String getExtendedKeyUsage(){
		try{
			if(x509Cert!=null){
				String allNames="";
				for(String s:x509Cert.getExtendedKeyUsage()){
					if(extendedKeyUsage.containsKey(s)){
						if(allNames!=""){allNames+=", ";}
						allNames+=extendedKeyUsage.get(s);
					}else{
						if(allNames!=""){allNames+=", ";}
						allNames+=s;
					}
				}
				if(allNames==""){return "-";}	/*Empty string as return value can cause troubles on Linux ==> return "-"*/
				return allNames;
			}
			return "-";
		}
		catch(Exception ex){ //TODO: catch-block verbessern
			return "-";
		}
	}
}
